from pathlib import Path
from threading import Lock

from test_pioneer.utils.exception.exceptions import ProjectException
from test_pioneer.utils.exception.tags import cant_save_yaml_error
from test_pioneer.logging.loggin_instance import test_pioneer_logger
from test_pioneer.project.template.template import template_1_str


def create_dir(dir_name: str) -> None:
    """
    :param dir_name: create dir use dir name
    :return: None
    """
    Path(dir_name).mkdir(
        parents=True,
        exist_ok=True
    )


def create_template(parent_name: str, project_path: str = None) -> None:
    if project_path is None:
        project_path: str = str(Path.cwd())
    template_dir = Path(project_path + "/" + parent_name)
    lock = Lock()
    if template_dir.exists() and template_dir.is_dir():
        lock.acquire()
        try:
            with open(str(template_dir) + "/" + parent_name + ".yml", "w+") as file_to_write:
                file_to_write.write(template_1_str)
        except ProjectException:
            raise ProjectException(cant_save_yaml_error)
        finally:
            lock.release()


def create_template_dir(project_path: str = None, parent_name: str = ".TestPioneer") -> None:
    test_pioneer_logger.info(f"create_template_dir, project_path: {project_path}, parent_name: {parent_name}")
    if project_path is None:
        project_path: str = str(Path.cwd())
    create_dir(project_path + "/" + parent_name)
    create_template(parent_name)



import json
import os
from pathlib import Path

from test_pioneer.executor.run.utils import select_with_runner
from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger


def run(step: dict, enable_logging: bool) -> bool:
    check_with_data = select_with_runner(step, enable_logging=enable_logging, mode="run")
    if not check_with_data[0]:
        return False
    else:
        execute_with = check_with_data[1]
    file = step.get("run")
    file = str(Path(os.getcwd() + file).absolute())
    if file is None:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"run param need file path: {step.get('run')}")
        return False
    if (Path(file).is_file() is False) or not Path(file).exists():
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"This file not exists: {step.get('run')}")
        return False
    file = json.loads(Path(file).read_text())
    execute_with(file)
    return True


import time

import yaml

from test_pioneer.utils.exception.exceptions import WrongInputException, YamlException
from test_pioneer.executor.browser.url import open_url
from test_pioneer.executor.file.download import download_single_file
from test_pioneer.executor.program.external_program import open_program, close_program
from test_pioneer.executor.run.executor_run import run
from test_pioneer.executor.run.executor_run_folder import run_folder
from test_pioneer.executor.test_recorder.logger import set_logger
from test_pioneer.executor.time.wait import blocked_wait
from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger
from test_pioneer.process.process_manager import process_manager_instance
from test_pioneer.utils.package.check import is_installed


def execute_yaml(stream: str, yaml_type: str = "File"):
    recording = None
    recoder = None
    if yaml_type == "File":
        file = open(stream, "r").read()
        yaml_data = yaml.safe_load(stream=file)
    elif yaml_type == "String":
        yaml_data = yaml.safe_load(stream=stream)
    else:
        raise WrongInputException("Wrong input: " + repr(stream))
    # Pre-check data structure
    if not isinstance(yaml_data, dict):
        raise YamlException(f"Not a dict: {yaml_data}")

    # Pre-check save log or not
    enable_logging = set_logger(yaml_data=yaml_data)
    if is_installed(package_name="je_auto_control"):
        from test_pioneer.executor.test_recorder.video_recoder import set_recoder
        # Pre-check recording or not
        recording, recoder = set_recoder(yaml_data=yaml_data)

    try:
        # Pre-check jobs
        if "jobs" not in yaml_data.keys():
            raise YamlException("No jobs tag")
        if isinstance(yaml_data.get("jobs"), dict) is False:
            raise YamlException("jobs not a dict")

        # Pre-check steps
        steps = yaml_data.get("jobs").get("steps", None)
        if steps is None or len(steps) <= 0:
            raise YamlException("Steps tag is empty")

        pre_check_failed: bool = False

        # Pre-check the job name has a duplicate or not
        for step in steps:
            if step.get("name", None) is None:
                step_log_check(
                    enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
                    message=f"Step need name tag")
                break
            name = step.get("name")
            if name in process_manager_instance.name_set:
                step_log_check(
                    enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
                    message=f"job name duplicated: {name}")
                pre_check_failed = True
                break
            else:
                process_manager_instance.name_set.add(name)

        # Execute step action
        for step in steps:
            if pre_check_failed:
                break
            name = step.get("name")

            if "run" in step.keys():
                if not run(step=step, enable_logging=enable_logging):
                    break

            elif "run_folder" in step.keys():
                if run_folder(step=step, enable_logging=enable_logging, mode="run_folder") is False:
                    break

            elif "open_url" in step.keys():
                if open_url(step=step, enable_logging=enable_logging):
                    break

            elif "download_file" in step.keys():
                if download_single_file(step=step, name=name, enable_logging=enable_logging):
                    break

            elif "wait" in step.keys():
                if blocked_wait(step=step, enable_logging=enable_logging) is False:
                    break

            elif "open_program" in step.keys():
                if open_program(step=step, name=name, enable_logging=enable_logging) is False:
                    break

            elif "close_program" in step.keys():
                if close_program(step=step, enable_logging=enable_logging) is False:
                    break

    except Exception as error:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"Error: {repr(error)}")
        if is_installed(package_name="je_auto_control"):
            if recording and recoder is not None:
                recoder.set_recoding_flag(False)
                while recoder.is_alive():
                    time.sleep(0.1)
        raise error
    if is_installed(package_name="je_auto_control"):
        if recording and recoder is not None:
            recoder.set_recoding_flag(False)
            while recoder.is_alive():
                time.sleep(0.1)


from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger
from test_pioneer.process.execute_process import ExecuteProcess
from test_pioneer.process.process_manager import process_manager_instance


def open_program(step: dict, name: str, enable_logging: bool = False) -> bool:
    if not isinstance(step.get("open_program"), str):
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"The 'open_program' parameter is not an str type: {step.get('open_program')}")
        return False
    step_log_check(
        enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
        message=f"Open program: {step.get('open_program')}")

    redirect_stdout = None
    redirect_error = None

    if "redirect_stdout" in step.keys():
        if not isinstance(step.get("redirect_stdout"), str):
            step_log_check(
                enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
                message=f"The 'redirect_stdout' parameter is not an str type: {step.get('redirect_stdout')}")
            return False
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
            message=f"Redirect stdout to: {step.get('redirect_stdout')}")
        redirect_stdout = step.get("redirect_stdout")

    if "redirect_stderr" in step.keys():
        if not isinstance(step.get("redirect_stderr"), str):
            step_log_check(
                enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
                message=f"The 'redirect_stderr' parameter is not an str type: {step.get('redirect_stderr')}")
            return False
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
            message=f"Redirect stderr to: {step.get('redirect_stderr')}")
        redirect_error = step.get("redirect_stdout")

    execute_process = ExecuteProcess()
    process_manager_instance.process_dict.update({name: execute_process})

    if redirect_error:
        execute_process.redirect_stdout = redirect_stdout

    if redirect_error:
        execute_process.redirect_stderr = redirect_error

    execute_process.start_process(step.get("open_program"))
    return True


def close_program(step: dict, enable_logging: bool = False) -> bool:
    if not isinstance(step.get("close_program"), str):
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"The 'close_program' parameter is not an str type: {step.get('close_program')}")
        return False
    step_log_check(
        enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
        message=f"Close program: {step.get('close_program')}")
    program_that_need_close = step.get("close_program")
    process_manager_instance.close_process(program_that_need_close)
    return True
